import * as d3 from "d3";
export class GaugeChart {
    constructor() {
        this.CenterX = 0;
        this.CenterY = 0;
        this.TopX = 0;
        this.TopY = 0;
        this.LeftX = 0;
        this.LeftY = 0;
        this.RightX = 0;
        this.RightY = 0;
        this.needleHeight = 0;
        this.needleRadius = 3;
    }
    fillChart(config, basicChart) {
        let svg = basicChart.svg;
        let margin = config.chart.margin;
        let outer = Math.min((config.chart.height - margin.top - margin.bottom), (config.chart.width - margin.left - margin.right));
        let inner = outer - config.chart.strokeWidth;
        let left = (config.chart.width - margin.left - margin.right) / 2;
        // let left = config.chart.width / 2 - outer / 2;
        this.needleHeight = outer;
        let arc = d3.arc().innerRadius(inner).outerRadius(outer).startAngle(Math.PI / 2).endAngle(-Math.PI / 2);
        svg.append("path")
            .attr("d", arc)
            .attr("class", "gauge_arc")
            .attr("fill", "#ccc")
            .attr("stroke-width", 1)
            .attr("transform", "translate(" + left + "," + outer + ")");
        if (config.data.sections) {
            for (let s of config.data.sections) {
                let start = this.degToRad(this.percToDeg((s.start - config.data.min) / (config.data.max - config.data.min)) - 180) / 2;
                let end = this.degToRad(this.percToDeg((s.end - config.data.min) / (config.data.max - config.data.min)) - 180) / 2;
                let arc2 = d3.arc().innerRadius(inner).outerRadius(outer).startAngle(start).endAngle(end);
                svg.append("path")
                    .attr("d", arc2)
                    .attr("class", "gauge_arc")
                    .attr("fill", s.color)
                    .attr("stroke-width", 1)
                    .attr("transform", "translate(" + left + "," + outer + ")");
            }
        }
        // value / (max - min) * 100
        let thetaRad = this.percToRad((config.data.value - config.data.min) / (config.data.max - config.data.min)) / 2;
        this.CenterX = left;
        this.CenterY = outer;
        this.TopX = this.CenterX - this.needleHeight * Math.cos(thetaRad);
        this.TopY = this.CenterY - this.needleHeight * Math.sin(thetaRad);
        this.LeftX = this.CenterX - this.needleRadius * Math.cos(thetaRad - Math.PI / 2);
        this.LeftY = this.CenterY - this.needleRadius * Math.sin(thetaRad - Math.PI / 2);
        this.RightX = this.CenterX - this.needleRadius * Math.cos(thetaRad + Math.PI / 2);
        this.RightY = this.CenterY - this.needleRadius * Math.sin(thetaRad + Math.PI / 2);
        this.appendLine(config, basicChart);
        let needleG = svg.append("g")
            .attr("tranform", "translate(" + (margin.left + outer) + "," + (margin.top + outer + this.needleRadius) + ")")
            .attr('class', 'needle')
            .attr("fill", "black");
        needleG.append("path").attr('d', "M " + this.LeftX + ' ' + this.LeftY + ' L ' + this.TopX + ' ' + this.TopY + ' L ' + this.RightX + ' ' + this.RightY);
        this.appendLabel(config, basicChart);
    }
    appendLabel(config, basicChart) {
        basicChart.svg.append("text")
            .text(config.data.value)
            .attr("x", this.CenterX)
            .attr("y", this.CenterY + config.chart.fontSize * 1.2)
            .attr("text-anchor", "middle")
            .style("font-size", config.chart.fontSize + "px");
    }
    appendLine(config, basicChart) {
        let thetaRad = this.percToRad((config.data.must - config.data.min) / (config.data.max - config.data.min)) / 2;
        let deg = (this.percToDeg((config.data.must - config.data.min) / (config.data.max - config.data.min)) - 180) / 2;
        let margin = config.chart.margin;
        let outer = Math.min((config.chart.height - margin.top - margin.bottom), (config.chart.width - margin.left - margin.right));
        let left = (config.chart.width - margin.left - margin.right) / 2;
        let inner = outer - config.chart.strokeWidth;
        let needleHeight = outer + (outer - inner) * 0.1;
        let needleRadius = 1;
        let TopX = this.CenterX - needleHeight * Math.cos(thetaRad);
        let TopY = this.CenterY - needleHeight * Math.sin(thetaRad);
        let TextTopX = this.CenterX - (needleHeight + 5) * Math.cos(thetaRad);
        let TextTopY = this.CenterY - (needleHeight + 5) * Math.sin(thetaRad);
        let LeftX = this.CenterX - needleRadius * Math.cos(thetaRad - Math.PI / 2);
        let LeftY = this.CenterY - needleRadius * Math.sin(thetaRad - Math.PI / 2);
        let RightX = this.CenterX - needleRadius * Math.cos(thetaRad + Math.PI / 2);
        let RightY = this.CenterY - needleRadius * Math.sin(thetaRad + Math.PI / 2);
        let labelSize = config.data.labelSize ? config.data.labelSize : 10;
        let needleG = basicChart.svg.append("g")
            .attr("tranform", "translate(" + (margin.left + outer) + "," + (margin.top) + ")")
            .attr('class', 'needle')
            .attr("stroke-width", 5)
            .attr("stroke", "red");
        needleG.append("path").attr('d', "M " + LeftX + ' ' + LeftY + ' L ' + TopX + ' ' + TopY + ' L ' + RightX + ' ' + RightY);
        if (config.data.showMustLabel) {
            basicChart.svg.append("text")
                .text(config.data.must)
                .attr("transform", "translate(" + TextTopX + "," + TextTopY + "),rotate(" + deg + ")")
                // .attr("x", TopX)
                // .attr("y", TopY)
                .attr("text-anchor", "middle")
                .style("font-size", labelSize + "px")
                .attr("fill", "red");
        }
        let arc = d3.arc().innerRadius(-10).outerRadius(inner - (outer - inner) * 0.1).startAngle(Math.PI / 2).endAngle(-Math.PI / 2);
        basicChart.svg.append("path")
            .attr("d", arc)
            .attr("class", "gauge_arc")
            .attr("fill", "#fff")
            .attr("transform", "translate(" + left + "," + (outer + 2) + ")");
    }
    percToDeg(percentage) {
        return percentage * 360;
    }
    percToRad(percentage) {
        return this.degToRad(this.percToDeg(percentage));
    }
    degToRad(degree) {
        return degree * Math.PI / 180;
    }
}
//# sourceMappingURL=gaugechart.js.map