import * as d3 from "d3";
export class RadarChart {
    constructor() {
        this.features = ["A", "B", "C", "D", "E", "F"];
    }
    fillChart(config, basicChart) {
        let svg = basicChart.svg;
        let data = config.data.data;
        let line = d3.line()
            .x((d) => d.x)
            .y((d) => d.y);
        let colors = ["darkorange", "gray", "navy"];
        for (var i = 0; i < data.length; i++) {
            let d = data[i];
            let color = colors[i];
            let coordinates = this.getPathCoordinates(d, config);
            //draw the path element
            svg.append("path")
                .datum(coordinates)
                .attr("d", line)
                .attr("stroke-width", 3)
                .attr("stroke", color)
                .attr("fill", color)
                .attr("stroke-opacity", 1)
                .attr("opacity", 0.5);
        }
    }
    genFakeRadarData() {
        let data = [];
        let features = ["A", "B", "C", "D", "E", "F"];
        //generate the data
        for (var i = 0; i < 3; i++) {
            var point = {};
            //each feature will be a random number from 1-9
            features.forEach(f => point[f] = 1 + Math.random() * 8);
            data.push(point);
        }
        return data;
    }
    getPathCoordinates(data_point, config) {
        let coordinates = [];
        for (var i = 0; i < this.features.length; i++) {
            let ft_name = this.features[i];
            let angle = (Math.PI / 2) + (2 * Math.PI * i / this.features.length);
            coordinates.push(this.angleToCoordinate(angle, data_point[ft_name], config));
        }
        return coordinates;
    }
    angleToCoordinate(angle, value, config) {
        let radialScale = config.chart.radarConfig.radialScale;
        let x = Math.cos(angle) * radialScale(value);
        let y = Math.sin(angle) * radialScale(value);
        return { "x": config.chart.width / 2 + x, "y": config.chart.height / 2 - y };
    }
}
//# sourceMappingURL=radarchart.js.map