import * as d3 from "d3";
import { regressionLinear, regressionQuad, regressionLoess, regressionPoly } from '../../library/regression/index.js';
export class Regression {
    getLinearRegression(basicChart, config) {
        let linearRegression = regressionLinear()
            .x((d) => d.x)
            .y((d) => d.y)
            .domain([config.axis.x.min, config.axis.x.max]);
        let res = linearRegression(config.data.points);
        let x = basicChart.axis.x['scale'];
        let y = basicChart.axis.y['scale'];
        let svg = basicChart.svg;
        x.domain(d3.extent(config.data.points, (d) => d.x));
        y.domain(d3.extent(config.data.points, (d) => d.y));
        let line = d3.line()
            .x((d) => x(d[0]))
            .y((d) => y(d[1]));
        this.appendLine(config, basicChart, res, line);
    }
    getQuadRegression(basicChart, config) {
        let domainMin = config.axis.x.min - Math.abs(config.axis.x.min * 0.5);
        let domainMax = config.axis.x.max + Math.abs(config.axis.x.min * 0.5);
        let quadRegression = regressionQuad()
            .x((d) => d.x)
            .y((d) => d.y)
            .domain([domainMin, domainMax]);
        let res = quadRegression(config.data.points);
        let x = basicChart.axis.x['scale'];
        let y = basicChart.axis.y['scale'];
        let svg = basicChart.svg;
        x.domain(d3.extent(config.data.points, (d) => d.x));
        y.domain(d3.extent(config.data.points, (d) => d.y));
        let line = d3.line()
            .x((d) => x(d[0]))
            .y((d) => y(d[1]));
        this.appendLine(config, basicChart, res, line);
    }
    getQuadExample(body) {
        let dataLinear = [{
                x: 8,
                y: 3
            }, {
                x: 2,
                y: 10
            }, {
                x: 11,
                y: 3
            }, {
                x: 6,
                y: 2
            }, {
                x: 5,
                y: 2
            }, {
                x: 4,
                y: 12
            }, {
                x: 14,
                y: 1
            }, {
                x: 9,
                y: 4
            }, {
                x: 6,
                y: 9
            }, {
                x: 1,
                y: 16
            }];
        let quadRegression = regressionQuad()
            .x((d) => d.x)
            .y((d) => d.y)
            .domain([-1.7, 16]);
        let res = quadRegression(dataLinear);
        let margin = {
            top: 30,
            right: 20,
            bottom: 30,
            left: 50
        }, width = 600 - margin.left - margin.right, height = 270 - margin.top - margin.bottom;
        let x = d3.scaleLinear().range([0, width]);
        let y = d3.scaleLinear().range([height, 0]);
        let svg = d3.select(body)
            .append("svg")
            .attr("width", width + margin.left + margin.right)
            .attr("height", height + margin.top + margin.bottom)
            .append("g")
            .attr("transform", "translate(" + margin.left + "," + margin.top + ")");
        x.domain(d3.extent(dataLinear, (d) => d.x));
        y.domain(d3.extent(dataLinear, (d) => d.y));
        // Add the valueline path.
        svg.selectAll("circle")
            .data(dataLinear)
            .enter()
            .append("circle")
            .attr("r", 5)
            .style("fill", "steelblue")
            .attr("cx", (d) => x(d.x))
            .attr("cy", (d) => y(d.y));
        let line = d3.line()
            .x((d) => x(d[0]))
            .y((d) => y(d[1]));
        svg.append("path")
            .datum(res)
            .attr("d", line)
            .style("fill", "none")
            .style("stroke", "steelblue")
            .style("stroke-width", "2px");
        svg.append("g")
            .attr("transform", "translate(0," + height + ")")
            .call(d3.axisBottom(x));
        svg.append("g")
            .call(d3.axisLeft(y));
    }
    getLoessRegression(basicChart, config) {
        let loessRegression = regressionLoess()
            .x((d) => d.x)
            .y((d) => d.y)
            .bandwidth(config.chart.regression.bandwidth);
        let res = loessRegression(config.data.points);
        let x = basicChart.axis.x['scale'];
        let y = basicChart.axis.y['scale'];
        let svg = basicChart.svg;
        x.domain(d3.extent(config.data.points, (d) => d.x));
        y.domain(d3.extent(config.data.points, (d) => d.y));
        let line = d3.line()
            .x((d) => x(d[0]))
            .y((d) => y(d[1]));
        this.appendLine(config, basicChart, res, line);
    }
    getPolyRegression(basicChart, config) {
        let polyRegression = regressionPoly()
            .x((d) => d.x)
            .y((d) => d.y)
            .order(config.chart.regression.order);
        let res = polyRegression(config.data.points);
        let x = basicChart.axis.x['scale'];
        let y = basicChart.axis.y['scale'];
        let svg = basicChart.svg;
        x.domain(d3.extent(config.data.points, (d) => d.x));
        y.domain(d3.extent(config.data.points, (d) => d.y));
        let line = d3.line()
            .x((d) => x(d[0]))
            .y((d) => y(d[1]));
        this.appendLine(config, basicChart, res, line);
    }
    appendLine(config, basicChart, res, line) {
        basicChart.svg.append("path")
            .datum(res)
            .attr("d", line)
            .style("fill", "none")
            .style("stroke", config.chart.regression.stroke)
            .style("stroke-width", config.chart.regression.width);
    }
}
//# sourceMappingURL=regression.js.map