import * as d3 from "d3";
export class LegendService {
    generateLegend(config, basicChart) {
        if (!config.chart.legend || !config.chart.legend.show) {
            return;
        }
        let data = JSON.parse(JSON.stringify(config.data));
        data = data.shift();
        data.shift();
        const keys = [];
        for (let d of data) {
            keys.push(config.names[d]);
        }
        var color = d3.scaleOrdinal()
            .domain(keys)
            .range(config.chart.colorPalette ? config.chart.colorPalette : d3.schemeSet2);
        const legendSelector = config.chart.legend.position === "bottom" ? '#legendBottom' : '#legendRight';
        let fontSize = config.chart.fontSize;
        let width = config.chart.width - config.chart.margin.left - config.chart.margin.right;
        let borderRadius = config.chart.legend.type === "round" ? "50" : "0";
        let l = `<div style="display: flex; flex-direction: row; flex-wrap: wrap; width: ${width}px; 
        font-size: ${fontSize}px; margin-left: ${config.chart.margin.left}px; margin-right: ${config.chart.margin.right}px; ">`;
        for (let key of keys) {
            l += `<div style="display: flex; margin: 2px 4px; align-items: center;">
            <div style="width: ${fontSize}px; height: ${fontSize}px; background-color: ${color(key)}; flex: 0 0 auto; border-radius: ${borderRadius}%; "></div>
            <div style="color: ${color(key)}; margin-left: 4px;">${key}</div>
            </div>`;
        }
        l += '</div>';
        let legend = d3.select(basicChart.body).select(legendSelector)
            .append('div').html(l);
        return;
    }
}
//# sourceMappingURL=legend.service.js.map