var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var ChartBuilder = function () { };
import jsdom from "jsdom";
const { JSDOM } = jsdom;
import * as c3mod from "./libs/c3-mod.js";
export function build(data, serverUrl, reqBodyData) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise(resolve => {
            var chartData = JSON.parse(JSON.stringify(data));
            var chartType = chartData.config.charttype;
            var serverVersion = chartData.config.serverVersion.replace(/[^a-zA-Z0-9\*\/\: \[\]-]/g, '');
            var htmlStub = '<html><head></head><body>' +
                '<div id="svg"></div>' +
                '<script type="text/javascript">window.onerror = function (message) { console.error(message); }</script>' +
                '<script type="text/javascript">var chartData=' + JSON.stringify(chartData) + ';</script>';
            if (chartData.config.charttype === 'c3') {
                htmlStub;
                console.log("c3");
            }
            else if (chartData.config.charttype === 'radar') {
                htmlStub += '<script src="libs/d3-radarchart.js"></script>';
            }
            htmlStub += '</body></html>';
            if (chartType === 'c3') {
                delete chartData.config;
            }
            var virtualConsole = new jsdom.VirtualConsole().sendTo(console);
            if (process.env.debug) {
                virtualConsole.on("warn", function (e) {
                    console.log(new Date().toISOString() + ' => JSDOM: [WARN]: ', e);
                });
                virtualConsole.on("log", function (e) {
                    console.log(new Date().toISOString() + ' => JSDOM: [LOG]: ', e);
                });
            }
            virtualConsole.on("error", function (e) {
                if (process.env.debug) {
                    console.log(new Date().toISOString() + ' => JSDOM: [ERROR]: ', e);
                }
                resolve({
                    'svg': 'ERROR: ' + e + '<!-- DEBUG DATA: ' + JSON.stringify(data) + '-->'
                });
            });
            virtualConsole.on("info", function (message) {
                if (message === 'onrendered') {
                    onrendered(this);
                }
            });
            virtualConsole.dom = new JSDOM(htmlStub, {
                virtualConsole: virtualConsole,
                runScripts: "dangerously",
                resources: "usable",
                includeNodeLocations: true,
                pretendToBeVisual: true,
                url: "http://localhost:8888"
            });
            console.log("dom created");
            console.log(virtualConsole.dom.window);
            c3mod.build(virtualConsole.dom.window);
            function onrendered(vconsole) {
                console.log("rendered");
                var domSVG = vconsole.dom.window.svgString;
                // var fs = require('fs');
                // var style = '<style type="text/css">\n /* <![CDATA[ */\n' +
                //     fs.readFileSync(__dirname + '/css/c3-mod.css', 'utf8')
                //     .replace(/[\n\r\t]/g, '')
                //     .replace(/\:\s*/g, ':')
                //     .replace(/\/\*.*?\*\//g, '')
                //     .replace(/\}/g, '} ') +
                //     '\n /* ]]> */\n</style>';
                var style = "";
                var re;
                if (chartType === 'radar') {
                    re = new RegExp(/\<\/svg\>$/);
                }
                else {
                    re = new RegExp(/\<\/svg\>/);
                }
                //domSVG = domSVG.replace(/clip-path/g, "clip-path");
                domSVG = domSVG.replace(re, style + '</svg>');
                domSVG = domSVG.replace(/\<svg /, '<svg serverVersion="' + serverVersion + '"' +
                    'xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" version="1.1" ');
                if (chartData.debug === true) {
                    domSVG += '<!-- DEBUG DATA: ' + JSON.stringify(data) + '-->';
                    domSVG += '<!-- DEBUG REQBODYDATA: ' + JSON.stringify(reqBodyData) + '-->';
                }
                if (process.env.debug && domSVG.length < 30) {
                    console.log(new Date().toISOString() + ' ---------------- INPUT DATA ----------------');
                    console.log(JSON.stringify(data));
                    console.log(new Date().toISOString() + ' ------------------ domSVG ------------------');
                    console.log(domSVG.substr(0, 500));
                    console.log(new Date().toISOString() + ' --------------------------------------------');
                }
                if (data.config.doctype !== 'html') {
                    var svgdata = '<?xml version="1.0" standalone="no" encoding="utf-8"?>\n<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">\n';
                    if (data.config.doctype !== 'svg') {
                        fs.writeFileSync('download/bar-' + data.config.uuid + '.svg', svgdata + domSVG, 'utf-8', function (err) {
                            if (err)
                                throw err;
                        });
                        resolve(require('./imagebuilder').build({
                            'filename': 'download/bar-' + data.config.uuid + '.svg',
                            'name': 'c3chart.svg'
                        }, data.config, true));
                    }
                    else {
                        resolve({
                            'svg': svgdata + domSVG
                        });
                    }
                }
                else {
                    resolve(domSVG);
                }
            }
        });
    });
}
;
// module.exports = new ChartBuilder();
export default ChartBuilder;
//# sourceMappingURL=chartBuilder.js.map