import { d3 } from "d3";
import * as RadarChart from "./libs/radar-chart.js";
var d3_locale_deDE = d3.locale({
    "decimal": ",",
    "thousands": ".",
    "grouping": [3],
    // "currency" : [ "", chartData.axis.y.tick.unit],
    "dateTime": "%a %b %e %X %Y",
    "date": "%d. %m. %Y",
    "time": "%H:%M:%S",
    "periods": ["AM", "PM"],
    "days": ["Sonnatg", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag"],
    "shortDays": ["So", "Mo", "Di", "Mi", "Do", "Fr", "Sa"],
    "months": ["Januar", "Februar", "März", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober",
        "November", "Dezember"],
    "shortMonths": ["Jan", "Feb", "Mer", "Apr", "Mai", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dez"]
});
d3.format = d3_locale_deDE.numberFormat;
window.d3 = d3;
function getD3Format(format) {
    var d3Format;
    switch (format[0]) {
        case 'N':
            d3Format = d3.format(",."
                + parseInt(format.substring(1))
                + "f");
            break;
        case 'P':
            // d3Format = d3.format("%,." + n + "f");
            d3Format = function (v) {
                return v === null ? '' : v.toFixed(parseInt(format.substring(1))) + "%";
            };
            break;
        default:
            d3Format = d3.format(",.0f");
            break;
    }
    return d3Format;
}
function getColor(series) {
    if (typeof chartData.legend.rows[series] != "undefined" && typeof chartData.legend.rows[series].color != "undefined") {
        return chartData.legend.rows[series].color;
    }
    else {
        return colorscale(series);
    }
}
var lineHeight = chartData.config.fontSize * 1.3;
if (chartData.config.legend) {
    var hasTitle = chartData.legend.title.trim() !== '';
    var rows = [];
    var legendWidth = 0;
    for (var i in chartData.legend.rows) {
        if (chartData.legend.rows[i].visible) {
            row = chartData.legend.rows[i];
            row.index = i;
            rows.push(row);
            if (row.label !== undefined && row.label !== null) {
                var labelWidth = getTextWidth(row.label, chartData.config.fontSize, chartData.config.fontFamily);
                legendWidth = legendWidth < labelWidth ? labelWidth : legendWidth;
            }
        }
    }
    var legendHeight = hasTitle ? 2 * lineHeight : 0;
    if (chartData.legend.title !== undefined && chartData.legend.title !== null) {
        var labelWidth = getTextWidth(chartData.legend.title.toString(), chartData.config.fontSize, chartData.config.fontFamily);
        legendWidth = legendWidth < labelWidth ? labelWidth : legendWidth;
    }
    legendWidth += chartData.config.radius + 50;
    if (chartData.config.legendposition === 'right') {
        chartData.config.w -= legendWidth;
        chartData.config.ExtraWidthX = chartData.config.ExtraWidthX === undefined
            ? legendWidth
            : chartData.config.ExtraWidthX + legendWidth;
        var line = 0;
        rows.forEach(function (item) {
            item['col'] = chartData.config.radius;
            item['line'] = line;
            line++;
        });
        legendHeight += line * lineHeight;
    }
    else {
        var line = 0;
        var col = 0;
        var lineWidth = 0;
        rows.forEach(function (item) {
            item['line'] = line;
            item['col'] = lineWidth;
            lineWidth += legendWidth;
            if (lineWidth > (chartData.config.w + chartData.config.ExtraWidthX)) {
                line++;
                lineWidth = 0;
                col = 0;
                item['line'] = line;
                item['col'] = lineWidth;
                lineWidth += legendWidth;
            }
        });
        legendHeight += (2 + line) * lineHeight;
        chartData.config.h -= legendHeight;
        chartData.config.ExtraWidthY = chartData.config.ExtraWidthY === undefined
            ? legendHeight
            : chartData.config.ExtraWidthY + legendHeight;
    }
}
var maxValue = chartData.config.maxValue;
var minValue = 0;
chartData.data.forEach(function (row) {
    row.forEach(function (col) {
        maxValue = Math.max(maxValue, col.value);
        minValue = Math.min(minValue, col.value);
    });
});
maxValue -= chartData.config.minValue;
chartData.config.maxValue = maxValue;
var factor = Math.abs(maxValue) + Math.abs(minValue);
chartData.data.forEach(function (row, rowIndex) {
    row.forEach(function (col) {
        col.id = chartData.legend.rows[rowIndex] !== undefined ? chartData.legend.rows[rowIndex].label : rowIndex;
        col.index = rowIndex;
        col.absValue = col.value;
        col.value = (Math.max(col.value - chartData.config.minValue, 0) + Math.abs(minValue)) / factor;
    });
});
/*
 * Render Chart
 */
RadarChart.draw('#svg', chartData.data, chartData.config);
var chartSvg = d3.select('#svg').selectAll('svg');
var chartSvgWidth = parseInt(chartSvg.attr('width'));
var chartSvgHeight = parseInt(chartSvg.attr('height'));
chartSvg.attr("viewBox", '0 0 ' + chartSvgWidth + ' ' + chartSvgHeight);
/*
 * Legend
 */
if (chartData.config.legend) {
    var xt = 20;
    var yt = 10;
    var xl = 0;
    var yl = 0;
    switch (chartData.config.legendposition) {
        case "right":
            xs = chartSvgWidth - legendWidth + chartData.config.TranslateX;
            ys = 20;
            break;
        default:
            xl = chartData.config.radius;
            xs = 0;
            ys = chartSvgHeight + (!isNaN(chartData.config.TranslateY) ? parseInt(chartData.config.TranslateY) : 0)
                - legendHeight;
            break;
    }
    var svg = chartSvg.append('svg')
        .attr("width", chartSvgWidth)
        .attr("height", legendHeight)
        .attr("viewBox", '0 0 ' + chartSvgWidth + ' ' + legendHeight)
        .attr("x", xs)
        .attr("y", ys)
        .append('g');
    var colorscale = d3.scale.category10();
    // Create the title for the legend
    if (chartData.legend.title.trim() !== '') {
        yl = lineHeight;
        var text = svg
            .append('g')
            .attr("class", "title")
            .append("text")
            .attr("y", yt)
            .attr("x", xt)
            .attr("text-anchor", "middle")
            .style("font-family", "sans-serif")
            .style("font-weight", "bold")
            .style("font-family", chartData.config.fontFamily + ',sans-serif')
            .style("font-size", chartData.config.fontSize + 'px')
            .text(chartData.legend.title);
    }
    // Initiate Legend
    var legend = svg.append("g")
        .attr("class", "legend")
        .attr("transform", "translate(" + xl + "," + yl + ")");
    // Create colour circle
    legend.selectAll('circle')
        .data(rows)
        .enter()
        .append("circle")
        .attr("r", chartData.config.radius)
        .attr("cx", function (d) {
        return d.col;
    })
        .attr("cy", function (d) {
        return (d.line + 1) * lineHeight - chartData.config.radius;
    })
        .attr("data-id", function (d, x) {
        return x;
    })
        .attr("width", 10)
        .attr("height", 10)
        .style("fill", function (d) {
        return getColor(d.index);
    });
    // Create text next to squares
    legend.selectAll('text')
        .data(rows)
        .enter()
        .append("text")
        .style("font-family", chartData.config.fontFamily + ',sans-serif')
        .style("font-size", chartData.config.fontSize + 'px')
        .style("font-weight", function (r) {
        return r.weight;
    })
        //.attr("text-anchor", "left")
        .attr("x", function (d) {
        return d.col + chartData.config.radius + 5;
    })
        .attr("data-id", function (d, x) {
        return x;
    })
        .attr("y", function (d) {
        return (d.line + 1) * lineHeight;
    })
        .attr("fill", "#737373")
        .text(function (d) {
        return d.label;
    });
}
function getTextWidth(text, fontSize, font) {
    var count = 0;
    text = text.toString();
    var textLength = text.length;
    if (textLength < 1) {
        return 0;
    }
    count += (textLength - text.replace(/[ijl]/g, "").length) * 0.23;
    count += (textLength - text.replace(/f/g, "").length) * 0.27;
    count += (textLength - text.replace(/[tI]/g, "").length) * 0.28;
    count += (textLength - text.replace(/r/g, "").length) * 0.34;
    count += (textLength - text.replace(/1/g, "").length) * 0.49;
    count += (textLength - text.replace(/[cksvxyzJ]/g, "").length) * 0.5;
    count += (textLength - text.replace(/[abdeghnopquL023456789]/g, "").length) * 0.56;
    count += (textLength - text.replace(/[FTZ]/g, "").length) * 0.61;
    count += (textLength - text.replace(/[ABEKPSVXY]/g, "").length) * 0.67;
    count += (textLength - text.replace(/[wCDHNRU]/g, "").length) * 0.73;
    count += (textLength - text.replace(/[GOQ]/g, "").length) * 0.78;
    count += (textLength - text.replace(/[mM]/g, "").length) * 0.84;
    count += (textLength - text.replace(/W/g, "").length) * 0.95;
    count += (textLength - text.replace(/[ \.]/g, "").length) * 0.28;
    text = text.replace(/[ \.]/g, "");
    count += (text.replace("/[a-zA-Z0-9]/gi", "").length) * 0.3;
    return Math.ceil(adjustFont(count * fontSize, font));
}
function adjustFont(textWidth, font) {
    var modifier = 1;
    switch (font.toLowerCase()) {
        case 'helvetica':
            modifier = 0.73;
            break;
        case 'calibri':
            modifier = 0.59;
            break;
        case 'times':
            modifier = 0.685;
            break;
        case 'serif':
            modifier = 0.92;
            break;
        case 'courier':
            modifier = 0.8;
            break;
    }
    return Math.ceil(textWidth * modifier);
}
window.svgString = d3.select("#svg").html();
console.info('onrendered');
//# sourceMappingURL=chart_radar_jsdom_scritp.js.map