var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var Chartcis = function () { };
import * as ChartBuilder from "./chartBuilder.js";
import { v4 as uuidv4 } from 'uuid';
/**
 *
 * @param data
 * @returns {*}
 */
export function build(data, serverUrl, reqBodyData) {
    return __awaiter(this, void 0, void 0, function* () {
        //var dataString = JSON.stringify(data);
        //dataString = dataString.replace(/([\u00A1-\uFFFF])/g, function (sub) {
        //    console.log(sub, sub.charCodeAt(0), sub.charCodeAt(0).toString(16));
        //    return '&#x' + sub.charCodeAt(0).toString(16) + ';';
        //});
        //
        //data = JSON.parse(dataString);
        data.config.uuid = uuidv4();
        let chart;
        switch (data.config.charttype) {
            case 'radar':
                chart = yield this.buildChart(this.buildRadarData(data), serverUrl);
                break;
            case 'c3':
            case 'bar':
                chart = yield this.buildChart(this.normalizeData(data), serverUrl, reqBodyData);
                break;
            default:
        }
        return chart;
    });
}
;
/**
 *
 * @param data
 * @returns {*}
 */
export function buildChart(data, serverUrl, reqBodyData) {
    return __awaiter(this, void 0, void 0, function* () {
        // await chart rendering and return it
        const chart = yield ChartBuilder.build(data, serverUrl, reqBodyData);
        return chart;
    });
}
;
/**
 *
 * @param data
 */
export function buildRadarData(data) {
    var radarData = {
        config: {
            doctype: 'svg',
            charttype: 'radar',
            chartview: "circle",
            autoConfig: true,
            w: 600,
            h: 600,
            factor: 1,
            factorLegend: 0.85,
            levels: 10,
            levelLegend: true,
            format: ',.2f',
            unit: '%',
            opacityArea: 0.4,
            legend: true,
            legendposition: 'bottom',
            ToRight: 5,
            shiftX: 80,
            shiftY: 40,
            circles: true,
            radius: 5,
            TranslateX: 40,
            serverVersion: '',
            ExtraWidthX: 100,
            ExtraWidthY: 100,
            separator: ",",
            decimal: "."
        },
        legend: {
            title: '',
            rows: []
        },
        data: []
    };
    if (data.debug) {
        radarData.serverData = data;
    }
    radarData.debug = data.debug;
    radarData.config.chartview = data.config.chartview || radarData.config.chartview;
    radarData.config.serverVersion = data.config.serverVersion.replace(/[^a-zA-Z0-9\*\/\: \[\]-]/g, '');
    radarData.config.legend = data.legend.show;
    radarData.config.levels = data.levels;
    radarData.config.legendposition = data.legend.position;
    radarData.config.w = data.size.width - radarData.config.ExtraWidthX;
    radarData.config.h = data.size.height - radarData.config.ExtraWidthY;
    radarData.config.fontSize = data.font.size.label;
    radarData.config.fontSizeChart = data.font.size.label;
    radarData.config.fontFamily = data.font.family;
    radarData.config.opacityArea = data.config.opacityArea;
    radarData.config.unit = data.unit;
    radarData.config.minValue = data.config.minValue || 0;
    radarData.config.maxValue = data.config.maxValue || 0;
    radarData.config.message = data.message;
    radarData.config.TranslateX = data.config.TranslateX || radarData.config.TranslateX;
    radarData.config.TranslateY = data.config.TranslateY || radarData.config.TranslateY;
    radarData.legend.title = data.title.text;
    if (data.axis && data.axis.y2 && data.axis.y2.format && data.axis.y2.format != "") {
        radarData.config.format = data.axis.y2.format;
    }
    if (data.format) {
        radarData.config.separator = data.format.separator;
        radarData.config.decimal = data.format.decimal;
    }
    var seriesCount = data.data.columns[0] && data.data.columns[0].length;
    for (var serieNr = 1; serieNr < seriesCount; serieNr++) {
        var serie = [];
        data.data.columns.forEach(function (col) {
            serie.push({
                axis: data.data.names[col[0]] !== undefined ? data.data.names[col[0]] : col[0],
                axisWeight: (data.data.fontWeight !== undefined ? data.data.fontWeight[col[0]] : 'normal'),
                value: col[serieNr]
            });
        });
        radarData.data.push(serie);
    }
    if (data.dataRowCol === 0) {
        for (var serieNr = 0; serieNr < seriesCount - 1; serieNr++) {
            var label = serieNr + 1;
            var weight = 'normal';
            if (data.axis.x.categories !== undefined &&
                data.axis.x.categories[serieNr] !== undefined &&
                data.axis.x.categories[serieNr] !== null) {
                label = data.axis.x.categories[serieNr].name;
                weight = data.axis.x.categories[serieNr].weight || 'normal';
            }
            radarData.legend.rows.push({
                id: serieNr,
                visible: true,
                label: label,
                weight: weight
            });
        }
    }
    else {
        data.data.namesOrder.forEach(function (name) {
            radarData.legend.rows.push({
                id: name,
                visible: true,
                label: data.data.names[name] !== undefined ? data.data.names[name] : name,
                weight: data.data.fontWeight[name] || 'normal'
            });
        });
    }
    return radarData;
}
/**
 *
 * @param res
 * @param file
 */
export function download(res, file) {
    var fs = require('fs');
    res.download(file.filename, file.name, function (err) {
        if (err)
            throw err;
        fs.unlinkSync(file.filename);
        fs = null;
    });
}
;
/**
 *
 * @param data
 * @returns {*}
 */
export function normalizeData(data) {
    var newChart = {
        'config': {
            'doctype': 'svg',
            'charttype': 'c3',
            'autoConfig': true
        },
        'charttype': 'bar',
        'title': {
            'text': '',
            'position': 'top-center',
            'padding': {
                'top': 0,
                'right': 0,
                'bottom': 0,
                'left': 0
            }
        },
        'size': {
            'width': '1000',
            'height': '500'
        },
        'font': {
            'size': 10
        },
        'interaction': {
            'enabled': true
        },
        'tooltip': {
            'show': false
        },
        'axis': {
            'rotated': false,
            'x': {
                'show': true,
                'label': {
                    'text': '',
                    'position': 'outer-right'
                },
                'tick': {
                    'rotate': 0
                }
            },
            'y': {
                'show': true,
                'min': '',
                'max': '',
                'label': {
                    'text': '',
                    'position': 'outer-top'
                },
                'tick': {
                    'format': '',
                    'rotate': 0,
                    'unit': ''
                }
            },
            'y2': {
                'show': false,
                'min': '',
                'max': '',
                'label': {
                    'text': '',
                    'position': 'outer-top'
                },
                'tick': {
                    'format': '',
                    'rotate': 0,
                    'unit': ''
                }
            }
        },
        'legend': {
            'show': true,
            'position': 'bottom'
        },
        'grid': {
            'x': {
                'show': false,
                'lines': [],
                'linevalues': {
                    'values': '',
                    'texts': '',
                    'position': 'middle'
                }
            },
            'y': {
                'show': false,
                'lines': [],
                'linevalues': {
                    'values': '',
                    'texts': '',
                    'position': 'middle'
                },
                'linevalues2': {
                    'values': '',
                    'texts': '',
                    'position': 'middle'
                }
            }
        },
        'point': {
            'show': true
        },
        'stacked': false,
        'fontSize': 11,
        'maxValue': '',
        'minValue': '',
        'unit': '',
        'divisor': '',
        'levels': 5,
        'dataRowCol': 0,
        'dataColAsLabel': "",
        'data': {
            'labels': {},
            'showLabels': false,
            'columns': [],
            'groups': [],
            'names': {},
            'axes': {}
        }
    };
    return this.MergeRecursive(data, newChart);
}
;
export function MergeRecursive(obj1, obj2) {
    for (var p in obj2) {
        if (obj1[p] === undefined) {
            obj1[p] = obj2[p];
        }
        else {
            if (typeof obj2[p] === "object") {
                obj1[p] = this.MergeRecursive(obj1[p], obj2[p]);
            }
            else {
                if (obj1[p] === undefined) {
                    obj1[p] = obj2[p];
                }
            }
        }
    }
    return obj1;
}
;
// module.exports = new Chartcis();
export default Chartcis;
//# sourceMappingURL=chartcis.js.map