var RadarChart = {
    draw: function (id, d, options) {
        var cfg = {
            radius: 5,
            w: 600,
            h: 600,
            factor: 1,
            factorLegend: .85,
            levels: 3,
            maxValue: 0,
            maxValuePercent: 1,
            radians: 2 * Math.PI,
            opacityArea: 0.5,
            ToRight: 5,
            TranslateX: 80,
            TranslateY: 30,
            ExtraWidthX: 100,
            ExtraWidthY: 100,
            color: d3.scale.category10(),
            message: '',
        };
        if ('undefined' !== typeof options) {
            for (var i in options) {
                if ('undefined' !== typeof options[i]) {
                    cfg[i] = options[i];
                }
            }
        }
        var allAxis = (d[0] && d[0].map(function (i, j) {
            return i;
        }) || []);
        var total = allAxis.length;
        var radius = cfg.factor * Math.min(cfg.w / 2, cfg.h / 2);
        var Format;
        if (!options.format || options.format == "" || options.format == ",.2f") {
            Format = d3.format(',.2f');
        }
        else {
            let local = {
                group: ".",
                decimal: ",",
            };
            if (options.separator) {
                local.group = options.separator;
            }
            if (options.decimal) {
                local.decimal = options.decimal;
            }
            numfmt.addLocale(local, "de-DE");
            Format = numfmt(options.format, { locale: 'de-DE' });
        }
        d3.select(id).select("svg").remove();
        var g = d3.select(id)
            .append("svg")
            .attr("width", cfg.w + cfg.ExtraWidthX)
            .attr("height", cfg.h + cfg.ExtraWidthY)
            .append("g")
            .attr("width", Math.min(cfg.w, cfg.h))
            .attr("height", Math.min(cfg.w, cfg.h))
            .attr("transform", "translate(" + cfg.TranslateX + "," + cfg.TranslateY + ")");
        //make chart homohenous
        cfg.w = cfg.h = Math.min(cfg.w, cfg.h);
        d3
            .select('svg')
            .append('text')
            .text(function () {
            return cfg.message;
        })
            .attr("text-anchor", "middle")
            .attr("x", cfg.w / 2 + cfg.TranslateX)
            .attr("y", cfg.w / 2 + cfg.TranslateY)
            .style("font-family", cfg.fontFamily + ",sans-serif")
            .style("font-size", cfg.fontSizeChart + 4 + "px");
        if (cfg.chartview === 'web') {
            //Web segments
            for (var j = 0; j < cfg.levels - 1; j++) {
                var levelFactor = cfg.factor * radius * ((j + 1) / cfg.levels);
                g.selectAll(".levels")
                    .data(allAxis)
                    .enter()
                    .append("svg:line")
                    .attr("x1", function (d, i) {
                    return levelFactor * (1 - cfg.factor * Math.sin(i * cfg.radians / total));
                })
                    .attr("y1", function (d, i) {
                    return levelFactor * (1 - cfg.factor * Math.cos(i * cfg.radians / total));
                })
                    .attr("x2", function (d, i) {
                    return levelFactor * (1 - cfg.factor * Math.sin((i + 1) * cfg.radians / total));
                })
                    .attr("y2", function (d, i) {
                    return levelFactor * (1 - cfg.factor * Math.cos((i + 1) * cfg.radians / total));
                })
                    .attr("class", "line")
                    .style("stroke", "grey")
                    .style("stroke-opacity", "0.3")
                    .attr("transform", "translate(" + (cfg.w / 2 - levelFactor) + ", " + (cfg.h / 2 - levelFactor) + ")");
            }
        }
        else {
            //Circular segments
            for (var j = 0; j < cfg.levels - 1; j++) {
                var levelFactor = cfg.factor * radius * ((j + 1) / cfg.levels);
                g.selectAll(".levels")
                    .data([1])
                    .enter()
                    .append("circle")
                    .attr("cx", cfg.w / 2)
                    .attr("cy", cfg.h / 2)
                    .attr("r", levelFactor)
                    .style("stroke", "grey")
                    .style('shape-rendering', '')
                    .style("stroke-opacity", "0.3")
                    .style("fill", "none");
            }
        }
        //Text indicating at what % each level is
        for (var j = 0; j < cfg.levels; j++) {
            var levelFactor = cfg.factor * radius * ((j + 1) / cfg.levels);
            g.selectAll(".levels")
                .data([1]) //dummy data
                .enter()
                .append("svg:text")
                .attr("x", function (d) {
                return levelFactor * (1 - cfg.factor * Math.sin(0));
            })
                .attr("y", function (d) {
                return levelFactor * (1 - cfg.factor * Math.cos(0));
            })
                .attr("class", "legend")
                .style("font-family", cfg.fontFamily + ",sans-serif")
                .style("font-size", cfg.fontSizeChart + "px")
                .attr("transform", "translate(" + (cfg.w / 2 - levelFactor + cfg.ToRight) + ", " + (cfg.h / 2 - levelFactor + cfg.fontSizeChart) + ")")
                .attr("fill", "#737373")
                .text(function () {
                return Format(((j + 1) * (cfg.maxValue) / cfg.levels) + cfg.minValue);
            });
        }
        series = 0;
        var axis = g.selectAll(".axis")
            .data(allAxis)
            .enter()
            .append("g")
            .attr("class", "axis");
        axis.append("line")
            .attr("x1", cfg.w / 2)
            .attr("y1", cfg.h / 2)
            .attr("x2", function (d, i) {
            return cfg.w / 2 * (1 - cfg.factor * Math.sin(i * cfg.radians / total));
        })
            .attr("y2", function (d, i) {
            return cfg.h / 2 * (1 - cfg.factor * Math.cos(i * cfg.radians / total));
        })
            .attr("class", "line")
            .style("stroke", function (axis) {
            return axis.axisColor || "gray";
        })
            .style("stroke-width", function (axis) {
            return axis.axisStrokeWidth || "1px";
        });
        axis.append("text")
            .attr("class", "legend")
            .text(function (d) {
            return d.axis;
        })
            .style("font-family", cfg.fontFamily + ",sans-serif")
            .style("font-size", cfg.fontSizeChart + "px")
            .style("font-weight", function (d) {
            return d.axisWeight || 'normal';
        })
            .attr("text-anchor", "middle")
            .attr("dy", "1em")
            .attr("x", function (d, i) {
            return cfg.w / 2 * (1 - cfg.factorLegend * Math.sin(i * cfg.radians / total)) - 60 * Math.sin(i * cfg.radians / total);
        })
            .attr("y", function (d, i) {
            var pos = cfg.h / 2 * (1 - Math.cos(i * cfg.radians / total));
            if (Math.ceil(pos) < Math.ceil(cfg.h / 2)) {
                pos -= cfg.fontSizeChart * 1.45;
            }
            return pos;
        });
        d.forEach(function (y, x) {
            dataValues = [];
            g.selectAll(".nodes")
                .data(y, function (j, i) {
                if (j.absValue || j.absValue === 0) {
                    dataValues.push([
                        cfg.w / 2 * (1 - (parseFloat(Math.max(j.value, 0)) / cfg.maxValuePercent) * cfg.factor * Math.sin(i * cfg.radians / total)),
                        cfg.h / 2 * (1 - (parseFloat(Math.max(j.value, 0)) / cfg.maxValuePercent) * cfg.factor * Math.cos(i * cfg.radians / total)),
                        j.id
                    ]);
                }
            });
            //dataValues.push(dataValues[0]);
            g.selectAll(".area")
                .data([dataValues])
                .enter()
                .append("polygon")
                .attr("class", "radar-chart-serie" + series)
                .attr("data-id", function () {
                return x;
            })
                .style("stroke-width", "2px")
                .style("stroke", cfg.color(series))
                .attr("points", function (d) {
                var str = "";
                for (var pti = 0; pti < d.length; pti++) {
                    str = str + d[pti][0] + "," + d[pti][1] + " ";
                }
                return str;
            })
                .style("fill", function (j, i) {
                return cfg.color(series);
            })
                .style("fill-opacity", cfg.opacityArea);
            series++;
        });
        series = 0;
        d.forEach(function (y, x) {
            g.selectAll(".nodes")
                .data(y)
                .enter()
                .append("svg:circle")
                .attr("data-id", function () {
                return x;
            })
                .attr("data-tooltip", function (d) {
                //console.info(d);
                //var dd = JSON.parse($$.pointData(d));
                //dd.label = $$.config.data_pairs.l;
                //dd.filterIdList = {};
                //dd.filterIdList[$$.config.data_names.l] = dd.id;
                return JSON.stringify({
                    id: d.id,
                    data: Format(d.absValue)
                });
            })
                .attr("class", "radar-chart-serie" + series)
                .attr('r', cfg.radius)
                .attr("alt", function (j) {
                return Math.max(j.value, 0);
            })
                .attr('visibility', function (j) {
                if (null === j.absValue || undefined === j.absValue) {
                    return 'hidden';
                }
                return 'visible';
            })
                .attr("cx", function (j, i) {
                return cfg.w / 2 * (1 - (Math.max(j.value, 0) / cfg.maxValuePercent) * cfg.factor * Math.sin(i * cfg.radians / total));
            })
                .attr("cy", function (j, i) {
                return cfg.h / 2 * (1 - (Math.max(j.value, 0) / cfg.maxValuePercent) * cfg.factor * Math.cos(i * cfg.radians / total));
            })
                .style("fill", cfg.color(series)).style("fill-opacity", .9);
            series++;
        });
        return g;
    }
};
//# sourceMappingURL=d3-radarchart.js.map