module.exports = {
    chart: function (id, data, style, d3) {
        var d = data.data;
        var cfg = {
            radius: 5,
            w: 600,
            h: 600,
            factor: 1,
            fontSize: 11,
            factorLegend: .85,
            levels: 3,
            levelLegend: true,
            maxValue: 0,
            minValue: 0,
            unit: '%',
            format: '.2f',
            legend: true,
            legendposition: 'bottom',
            radians: 2 * Math.PI,
            opacityArea: 0.5,
            ToRight: 5,
            TranslateX: 80,
            TranslateY: 30,
            shiftX: 80,
            shiftY: 40,
            color: d3.scale.category10()
        };
        function getPosition(i, range, factor, func) {
            factor = typeof factor !== 'undefined' ? factor : 1;
            return range * (1 - factor * func(i * cfg.radians / total));
        }
        function getHorizontalPosition(i, range, factor) {
            return getPosition(i, range, factor, Math.sin);
        }
        function getVerticalPosition(i, range, factor) {
            return getPosition(i, range, factor, Math.cos);
        }
        function getColor(series) {
            if (typeof data.legend.rows[series] != "undefined"
                && typeof data.legend.rows[series].color != "undefined") {
                return data.legend.rows[series].color;
            }
            else {
                return cfg.color(series);
            }
        }
        if ('undefined' !== typeof data.config) {
            for (var i in data.config) {
                if ('undefined' !== typeof data.config[i]) {
                    cfg[i] = data.config[i];
                }
            }
        }
        if (cfg.minValue > 0) {
            d.forEach(function (y, x) {
                y.forEach(function (x) {
                    x.value = x.value - cfg.minValue;
                });
            });
            cfg.maxValue = cfg.maxValue - cfg.minValue;
        }
        cfg.maxValue = Math.max(cfg.maxValue, d3.max(d, function (i) {
            return d3.max(i.map(function (o) {
                return o.value;
            }));
        }));
        var allAxis = (d[0].map(function (i, j) {
            return i.axis;
        }));
        var total = allAxis.length;
        var radius = cfg.factor * Math.min(cfg.w / 2, cfg.h / 2);
        var Format = d3.format(cfg.format);
        // var Format = numfmt('#,##0#.000')
        d3.select(id).select("svg").remove();
        d3.select(id)
            .append("svg")
            .attr("width", "100%")
            .attr("height", "100%")
            .attr("version", 1.1)
            .attr("xmlns", "http://www.w3.org/2000/svg");
        //			.attr("transform", "translate(" + cfg.TranslateX + "," + cfg.TranslateY + ")");
        d3.select(id).selectAll("svg").append("style").attr('type', 'text/css').text(style);
        var g = d3.select(id).selectAll("svg").append("g");
        var tooltip;
        //Circular segments
        for (var j = 0; j < cfg.levels; j++) {
            var levelFactor = cfg.factor * radius * ((j + 1) / cfg.levels);
            g.selectAll(".levels")
                .data(allAxis)
                .enter()
                .append("svg:line")
                .attr("x1", function (d, i) {
                return levelFactor * (1 - cfg.factor * Math.sin(i * cfg.radians / total)) + cfg.shiftX;
            })
                .attr("y1", function (d, i) {
                return levelFactor * (1 - cfg.factor * Math.cos(i * cfg.radians / total)) + cfg.shiftY;
            })
                .attr("x2", function (d, i) {
                return levelFactor * (1 - cfg.factor * Math.sin((i + 1) * cfg.radians / total)) + cfg.shiftX;
            })
                .attr("y2", function (d, i) {
                return levelFactor * (1 - cfg.factor * Math.cos((i + 1) * cfg.radians / total)) + cfg.shiftY;
            })
                .attr("class", "line")
                .style("stroke", "grey")
                .style("stroke-opacity", "0.75")
                .style("stroke-width", "0.3px")
                .attr("transform", "translate(" + (cfg.w / 2 - levelFactor) + ", " + (cfg.h / 2 - levelFactor) + ")");
        }
        //Text indicating at what % each level is
        if (cfg.levelLegend) {
            for (var j = 0; j < cfg.levels; j++) {
                var levelFactor = cfg.factor * radius * ((j + 1) / cfg.levels);
                g.selectAll(".levels")
                    .data([1]) //dummy data
                    .enter()
                    .append("svg:text")
                    .attr("x", function (d) {
                    return levelFactor * (1 - cfg.factor * Math.sin(0)) + cfg.shiftX;
                })
                    .attr("y", function (d) {
                    return levelFactor * (1 - cfg.factor * Math.cos(0)) + cfg.shiftY;
                })
                    .attr("class", "legend")
                    .style("font-family", "sans-serif")
                    .style("font-size", cfg.fontSize - 1)
                    .attr("transform", "translate(" + (cfg.w / 2 - levelFactor + cfg.ToRight) + ", " + (cfg.h / 2 - levelFactor) + ")")
                    .attr("fill", "#737373")
                    .text(Format(cfg.minValue + (j + 1) * cfg.maxValue / cfg.levels) + cfg.unit);
            }
        }
        series = 0;
        var axis = g.selectAll(".axis")
            .data(allAxis)
            .enter()
            .append("g")
            .attr("class", "axis");
        axis.append("line")
            .attr("x1", cfg.w / 2 + cfg.shiftX)
            .attr("y1", cfg.h / 2 + cfg.shiftY)
            .attr("x2", function (d, i) {
            return cfg.w / 2 * (1 - cfg.factor * Math.sin(i * cfg.radians / total)) + cfg.shiftX;
        })
            .attr("y2", function (d, i) {
            return cfg.h / 2 * (1 - cfg.factor * Math.cos(i * cfg.radians / total)) + cfg.shiftY;
        })
            .attr("class", "line")
            .style("stroke", "grey")
            .style("stroke-width", "1px");
        axis.append("text")
            .attr("class", "legend")
            .text(function (d) {
            return d;
        })
            .style("font-family", "sans-serif")
            .style("font-size", cfg.fontSize)
            .attr("text-anchor", function (d, i) {
            var p = getHorizontalPosition(i, 0.5);
            return ((p < 0.4) ? 'end' : ((p > 0.6) ? 'start' : 'middle'));
        })
            .attr("dy", "1em")
            .attr("transform", function (d, i) {
            return "translate(0, -10)";
        })
            .attr("x", function (d, i) {
            return cfg.w / 2 * (1 - cfg.factorLegend * Math.sin(i * cfg.radians / total)) - 60 * Math.sin(i * cfg.radians / total) + cfg.shiftX;
        })
            .attr("y", function (d, i) {
            return cfg.h / 2 * (1 - Math.cos(i * cfg.radians / total)) - 20 * Math.cos(i * cfg.radians / total) + cfg.shiftY;
        });
        d.forEach(function (y, x) {
            dataValues = [];
            g.selectAll(".nodes")
                .data(y, function (j, i) {
                dataValues.push([
                    cfg.w / 2 * (1 - (parseFloat(Math.max(j.value, 0)) / cfg.maxValue) * cfg.factor * Math.sin(i * cfg.radians / total)) + cfg.shiftX,
                    cfg.h / 2 * (1 - (parseFloat(Math.max(j.value, 0)) / cfg.maxValue) * cfg.factor * Math.cos(i * cfg.radians / total)) + cfg.shiftY
                ]);
            });
            dataValues.push(dataValues[0]);
            g.selectAll(".area")
                .data([dataValues])
                .enter()
                .append("polygon")
                .attr("class", "chart-serie radar-chart-serie" + series)
                .style("stroke-width", "2px")
                .style("stroke", getColor(series))
                .attr("points", function (d) {
                var str = "";
                for (var pti = 0; pti < d.length; pti++) {
                    str = str + d[pti][0] + "," + d[pti][1] + " ";
                }
                return str;
            })
                .style("fill", function (j, i) {
                return getColor(series);
            })
                .style("fill-opacity", cfg.opacityArea)
                .on('mouseover', function (d) {
                z = "polygon." + d3.select(this).attr("class");
                g.selectAll("polygon")
                    .transition(200)
                    .style("fill-opacity", 0.1);
                g.selectAll(z)
                    .transition(200)
                    .style("fill-opacity", .7);
            })
                .on('mouseout', function () {
                g.selectAll("polygon")
                    .transition(200)
                    .style("fill-opacity", cfg.opacityArea);
            });
            series++;
        });
        series = 0;
        d.forEach(function (y, x) {
            g.selectAll(".nodes")
                .data(y).enter()
                .append("svg:circle")
                .attr("class", "radar-chart-serie" + series)
                .attr('r', cfg.radius)
                .attr("alt", function (j) {
                return Math.max(j.value, 0);
            })
                .attr("cx", function (j, i) {
                dataValues.push([
                    cfg.w / 2 * (1 - (parseFloat(Math.max(j.value, 0)) / cfg.maxValue) * cfg.factor * Math.sin(i * cfg.radians / total)) + cfg.shiftX,
                    cfg.h / 2 * (1 - (parseFloat(Math.max(j.value, 0)) / cfg.maxValue) * cfg.factor * Math.cos(i * cfg.radians / total)) + cfg.shiftY
                ]);
                return cfg.w / 2 * (1 - (Math.max(j.value, 0) / cfg.maxValue) * cfg.factor * Math.sin(i * cfg.radians / total)) + cfg.shiftX;
            })
                .attr("cy", function (j, i) {
                return cfg.h / 2 * (1 - (Math.max(j.value, 0) / cfg.maxValue) * cfg.factor * Math.cos(i * cfg.radians / total)) + cfg.shiftY;
            })
                .attr("data-id", function (j) {
                return j.axis;
            })
                .style("fill", getColor(series)).style("fill-opacity", .9)
                .on('mouseover', function (d) {
                newX = parseFloat(d3.select(this).attr('cx')) - 10 + cfg.shiftX;
                newY = parseFloat(d3.select(this).attr('cy')) - 5 + cfg.shiftY;
                tooltip
                    .attr('x', newX)
                    .attr('y', newY)
                    .text(Format(d.value))
                    .transition(200)
                    .style('opacity', 1);
                z = "polygon." + d3.select(this).attr("class");
                g.selectAll("polygon")
                    .transition(200)
                    .style("fill-opacity", 0.1);
                g.selectAll(z)
                    .transition(200)
                    .style("fill-opacity", .7);
            })
                .on('mouseout', function () {
                tooltip
                    .transition(200)
                    .style('opacity', 0);
                g.selectAll("polygon")
                    .transition(200)
                    .style("fill-opacity", cfg.opacityArea);
            })
                .append("svg:title")
                .text(function (j) {
                return Math.max(j.value + cfg.minValue, 0);
            });
            series++;
        });
        //Tooltip
        tooltip = g.append('text')
            .style('opacity', 0)
            .style('font-family', 'sans-serif')
            .style('font-size', cfg.fontSize + 2);
        return g;
    }
};
//# sourceMappingURL=radar-chart.js.map