import { Bar, Chart } from "../../model";
import { Config } from "../../model/config";
import * as d3 from "d3";
import { LegendService } from "../legend.service.js";

export class BarChart {


    fillChart(config: Config, basicChart: any) {

        let data: any[] = config.data;
        let svg = basicChart.svg;

        return this.fillGroupedChart(config, basicChart);

        svg.selectAll("mybar")
            .data(data)
            .join("rect")
            .attr("x", function(d: any){ return basicChart.axis.x.scale(d['x']) })
            .attr("y", function(d: any){ return basicChart.axis[(d['yAxis'] === 'y'? 'y' : 'y1')].scale(0) - basicChart.axis[(d['yAxis'] === 'y'? 'y' : 'y1')].scale(d['y'])})
            // .attr("y", function(d: any){ return basicChart.axis[(d['yAxis'] === 'y'? 'y' : 'y1')].scale(d['y']) })
            .attr("width", (basicChart.axis.x.scale as any).bandwidth() / 7)
            .attr("height", function(d: any){ 
                let height = basicChart.axis[(d['yAxis'] === 'y'? 'y' : 'y1')].scale(d['y']);
                
                return height; 
             })
            .attr("fill", "#69b3a2")
    }

    fillGroupedChart(config: Config, basicChart: Chart) {
     
        let svg = basicChart.svg;

        const subgroups = config.data[0].filter((d: any) => d != "group");
        // console.log(subgroups);

        const groups = [];
        for (let index = 1; index < config.data.length; index++) { groups.push(config.data[index][0])}

        // console.log(groups);
        let bars = [];

        for (let outer = 1; outer < config.data.length; outer++) {
            // ['group', 'val']
            let bar: any = {}
            for (let index = 0; index < config.data[0].length; index++) {
                // ['group', 'val']
                
                bar[config.data[0][index]] = config.data[outer][index]
            }

            bars.push(bar)

        }

        
        let x = basicChart.axis.x.scale as any;
        let y = basicChart.axis.y.scale as any;
        x.padding(0.2)

        // scale für subgruppen
        const xSubGroups = d3.scaleBand().domain(subgroups).range([0, (basicChart.axis.x.scale as any).bandwidth()]).padding(0.05)
        
        let color: d3.ScaleOrdinal<string, unknown, never>;

        if (config.chart.colorPalette) {
            color = d3.scaleOrdinal().domain(subgroups).range(config.chart.colorPalette)
        } else {
            color = d3.scaleOrdinal().domain(subgroups).range(["#8EBC00", "#25A0DA", "#EB7A2A", "#D8E404", "#309B46", "#16ABA9", "#DC5B20", "#F2CA04", "#24C6D2", "#8A8065", "#8EBC00", "#25A0DA", "#EB7A2A", "#D8E404", "#309B46", "#16ABA9", "#DC5B20", "#F2CA04", "#24C6D2", "#8A8065"])
        }




        svg.append("g")
            .selectAll("g")
            .data(bars)
            .enter()
            .append("g")
                .attr("transform", (d: any) => {return "translate(" + x(d.group) + ",0)";})
            .selectAll("rect")
            .data((d: any) => { return subgroups.map((key: any) => {
                return {key: key, value: d[key]}
            })})
            .enter().append("rect")
                .attr("x", function(d: any) { return xSubGroups(d.key); })
                .attr("y", function(d: any) {return y(Math.max(d.value, 0)); })
                .attr("width", xSubGroups.bandwidth())
                .attr("height", function(d: any) { return Math.abs(y(d.value) - y(0))})
                .attr("fill", (d: any) => { return color(d.key) as string})



        if (config.chart.legend.show) {
            const legendService = new LegendService();
            legendService.generateLegend(config, basicChart)
        }
    }

    
}

