import { Chart } from "../../model/chart";
import { Config } from "../../model/config";
import * as d3 from "d3";


export class RadarChart {

    features: any[] = ["A","B","C","D","E","F"];

    fillChart(config: Config, basicChart: Chart) {
        let svg = basicChart.svg;
        let data = config.data.data;


        let line = d3.line()
            .x((d: any) => d.x)
            .y((d: any) => d.y);
        let colors = ["darkorange", "gray", "navy"];


        for (var i = 0; i < data.length; i ++){
            let d = data[i];
            let color = colors[i];
            let coordinates = this.getPathCoordinates(d, config);
        
            //draw the path element
            svg.append("path")
            .datum(coordinates)
            .attr("d",line as any)
            .attr("stroke-width", 3)
            .attr("stroke", color)
            .attr("fill", color)
            .attr("stroke-opacity", 1)
            .attr("opacity", 0.5);
        }


    }



    private genFakeRadarData() {
        let data = [];
        let features = ["A","B","C","D","E","F"];
        //generate the data
        for (var i = 0; i < 3; i++){
            var point: any = {}
            //each feature will be a random number from 1-9
            features.forEach(f => point[f] = 1 + Math.random() * 8);
            data.push(point);
        }
        return data;
    }

    private getPathCoordinates(data_point: any, config: Config){
        let coordinates = [];
        for (var i = 0; i < this.features.length; i++){
            let ft_name = this.features[i];
            let angle = (Math.PI / 2) + (2 * Math.PI * i / this.features.length);
            coordinates.push(this.angleToCoordinate(angle, data_point[ft_name], config));
        }
        return coordinates;
    }

    private angleToCoordinate(angle: any, value: any, config: Config){
        let radialScale =  config.chart.radarConfig.radialScale;
        let x = Math.cos(angle) * radialScale(value);
        let y = Math.sin(angle) * radialScale(value);
        return {"x": config.chart.width / 2 + x, "y": config.chart.height / 2 - y};
    }
    
}