import jsdom from "jsdom";
import { BasicChartService } from "../chart/basicchart.service.js";
import { BarChart } from "../chart/charttypes/barchart.js";
import { GaugeChart } from "../chart/charttypes/gaugechart.js";
import { PointChart } from "../chart/charttypes/pointchart.js";
import { RadarChart } from "../chart/charttypes/radarchart.js";
import { Regression } from "../chart/charttypes/regression.js";
import { Config } from "../model/config.js";
const { JSDOM } = jsdom;

export class Chart {
    basicChartService: BasicChartService = new BasicChartService();
    
    
    regression: Regression = new Regression();


    getPointChart(config: Config) {
        const dom = this.createDom(config);
        let basicChart;
        basicChart = this.basicChartService.getBasicChart(dom.window.document, config);
        let pointChart: PointChart = new PointChart();
        pointChart.fillChart(config, basicChart)
        return dom.serialize()
    }

    getBarChart(config: Config) {
        console.log("bar chart")
        const dom = this.createDom(config);
        let basicChart;
        basicChart = this.basicChartService.getBasicChart(dom.window.document, config);
        let barChart: BarChart = new BarChart();
        barChart.fillChart(config, basicChart)
        return dom.serialize();
    }

    getGaugeChart(config: Config) {
        const dom = this.createDom(config);
        let basicChart;
        basicChart = this.basicChartService.getBasicChart(dom.window.document, config);
        let gaugeChart = new GaugeChart();
        gaugeChart.fillChart(config, basicChart)
        return dom.serialize();
    }

    getRadarChart(config: Config) {
        const dom = this.createDom(config);
        let basicChart;
        basicChart = this.basicChartService.getBasicChart(dom.window.document, config);
        let radar = new RadarChart();
        radar.fillChart(config, basicChart);
        return dom.serialize();
    }




    private createDom(config: Config): jsdom.JSDOM {
        const padding = `padding: ${config.chart.margin.top}px ${config.chart.margin.right}px ${config.chart.margin.bottom}px ${config.chart.margin.left}px `;
        const html = `<!DOCTYPE html>
        <body>
        <div style="display: flex;">
            <div id="chart" style=""></div>
            <div id="legendRight"></div>
        </div>
        <div id="legendBottom">
        </div>
        </body>`;
        var virtualConsole = new jsdom.VirtualConsole().sendTo(console);
        const dom = new JSDOM(html,{ pretendToBeVisual: true });

    
        return dom;
    }
}